==> dddns-1.0.1.tar.gz: dddns-1.0.1/misc/bandlimit-failover.pl <==
#! /usr/bin/perl -w

$version = '$Id: bandlimit-failover.pl,v 1.9 2004/11/29 12:00:22 andrewm Exp $';
# (c) 2004 Leading Edge Business Solutions and Andrew McGill
# Distribution permitted under terms of GPL licence, as contained
# in the file COPYING
#
# Telkom bandlimit fail-over
#
# NOTE - DON'T RUN THIS AUTOMATICALLY EXCEPT FROM routeadjust.sh
# (you'll get strange behaviour)
#
# Telkom SA sells 3G of bandwidth for R250.00 (50USD).  After you
# pass this limit, you are bandlimited for "international"
# traffic.  This script checks whether you have passwd the limit
# and then brings up an additional interface on another ADSL
# account for international traffic only.
#
# Configuration files:
#
# /etc/ppp/pppoe.conf: main ppp0 account - never modified
# /etc/bandlimit-failover/account.conf: symbolic link to current account*.conf
# /etc/bandlimit-failover/account1.conf: first failover account for ppp1
# /etc/bandlimit-failover/account2.conf: second failover account for ppp1
# /etc/bandlimit-failover/account*.conf: additional failover accounts
#
# Usage:
# This script should be called in /etc/ppp/ip-up.d ... WITH A
# LOCK FILE ...  it is a real danger that the script will keep 
# bouncing links, especially if it runs multiple times.

no warnings qw{ uninitialized };

# We expect in our working directory a list of accounts
#
#	ppp0config	config for ppp0 - always in use
#	ppp1config	symbolic link to current ppp1 configuration - changed by this script
#	account1.conf	additional account details
#	account2.conf	additional account details
#	account3.conf	additional account details
#
# The account[123].conf files should share their PIDFILE setting
# to avoid running concurrently, and also so that account2 can
# shut down account1

$etc = "/etc/bandlimit-failover";
$DEBUG = "echo debug ...";
$DEBUG = '';

$ppp0config = "/etc/ppp/pppoe.conf";
$ppp1config = "$etc/active.conf";

# Set the file timestamp ...
sub touch($)
{
	utime undef,undef,(shift);
}

sub findnextaccount()
{
	@accountlist = sort glob("$etc/account*.conf");
	$activeaccount = readlink("$ppp1config");
	$nextaccount =  '';
	$usenext = 0;
	foreach $accountname (@accountlist) {
		if ($usenext) {
			$nextaccount = $accountname;
			$usenext = 0;
			last;
		}
		if ($activeaccount eq $accountname) {
			$usenext = 1;
		}
	}
	if ($nextaccount eq '' || $usenext) {
		# Rotate back to the first account
		$nextaccount = $accountlist[0];
	}
	system "ln -sf $nextaccount $ppp1config";
	($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
		$atime,$mtime,$ctime,$blksize,$blocks) = stat($nextaccount);
	$age = time() - $mtime;
	if ($age >= 0 && $age < 3600) {
		print "Skipping $ppp1config => $nextaccount (age $age seconds)\n";
		return 0;
	}
	else {
		print "Using config $ppp1config =>  $nextaccount\n";
		touch($nextaccount);
		return 1;
	}
}


# Get our IP
sub getpppipaddr($)
{
	my $interface = shift;
	my $look;
	@ifconfig = @ifconfig_orig;
	while ($_ = shift @ifconfig) {
		if ( m/^$interface\s/ ) { $look=1; }
		if ($look && m/inet addr:([\d.]+)/) {
			return $1;
		}
	}
	return '';
}


sub isbandlimited
{
	
	my @ppp= @_;
	# Telkom has 165.165.64.0/17 as their degraded network
	# Well - they did ...
	return (
		$ppp[0] == 165 && $ppp[1] == 165 && $ppp[2] <= 128
	);
}

# See what's up, what's down, and what's limited 
sub getpppstatus()
{
	@ifconfig_orig = qx{ ifconfig };
	$ppp0 = getpppipaddr("ppp0");
	@ppp0 = split /\./,$ppp0;
	$bandlimited0 = isbandlimited( @ppp0 );
	$ppp0status = $ppp0[0] ? ( $bandlimited0 ? "bandlimited" : "normal" ) : "down";

	$ppp1 = getpppipaddr("ppp1");
	@ppp1 = split /\./,$ppp1;
	$bandlimited1 = isbandlimited ($ppp1);
	$ppp1status = $ppp1[0] ? ( $bandlimited1 ? "bandlimited" : "normal" ) : "down";

	# Check if there is a ppp2 up - there shouldn't be, but
	# it is possible
	$ppp2 = getpppipaddr("ppp2");
	@ppp2 = split /\./,$ppp2;
	$bandlimited2 = isbandlimited($ppp2);
	$ppp2status = $ppp2[0] ? ( $bandlimited2 ? "bandlimited" : "normal" ) : "down";
}


# These are the states that are catered for 
#
# 0: no connection
# 1: single connection (ppp0, not bandlimited)
# 2: degraded connection (ppp0 which is bandlimited)
# 3: dual connection (ppp0 bandlimited, ppp1 no limit)
# 4: degraded dual connection (ppp0 andlimited, ppp1 bandlimited)
#
# We rely on the internals of the rp-pppoe adsl-start script to
# ensure that we don't run more than one interface simultaneously
# - specifically that it names the lock file after the symbolic
# link, rather than what that link points to.

getpppstatus();

$version =~ s/^.Id: //;
$version =~ s/,v//;
$version =~ s/ \d+:\d\d:\d\d.*//;
print "$version\n";
print "Before: ppp0 $ppp0 $ppp0status, ppp1 $ppp1 $ppp1status\n";

if ($ppp0 eq '') {
	if ($ppp1 ne '') {
		print "Strange, ppp1 active without ppp0 ... killing it all\n";
		print "ACTION: taking down ".readlink($ppp1config)." and $ppp0config\n";
		system "$DEBUG adsl-stop $ppp1config";
		system "$DEBUG adsl-stop $ppp0config";
	}
	# no connection
	print "State: No ppp connection\n";
	print "ACTION: bring ppp0 up\n";
	system "$DEBUG adsl-start $ppp0config";
}
elsif ($ppp1 eq '') {
	# Single connection
	if ($bandlimited0) {
		# Degraded connecton
		if (findnextaccount()) {
			print "ACTION: starting ppp1\n";
			system "$DEBUG adsl-start $ppp1config";  # fixme ... this needs to refer to a config file ...
		}
		else {
			print "ACTION: Nothing! No untried accounts left\n";
		}
	}
	else {
		print "$DEBUG Single, ppp0 $ppp0, not bandlimited (no action)\n";
		exit;
	}
}
else { # Both ppp0 and ppp1 are up
	# Dual connection
	if (! $bandlimited0) {
		print "ACTION: taking unnecessary ppp1 down\n";
		system "$DEBUG adsl-stop $ppp1config"; 
	}
	elsif ($bandlimited1) {
		# ppp1 has run out of juice - rotate to next link
		print "ACTION: rotating ".readlink($ppp1config)."\n";
		system "$DEBUG adsl-stop $ppp1config";

		if (findnextaccount()) {
			sleep(5);
			print "ACTION: starting ppp1\n";
			system "$DEBUG adsl-start $ppp1config";
			sleep(10); # don't be in a hurry ...
		}
		else {
			print "ACTION: Nothing! No accounts available!\n";
		}
	}
	else {
		print "Action: No change\n";
		exit; # Don't print status 
	}
}

getpppstatus();
print "After: ppp0 $ppp0 $ppp0status, ppp1 $ppp1 $ppp1status\n";

==> dddns-1.0.2.tar.gz: dddns-1.0.2/misc/bandlimit-failover.pl <==
#! /usr/bin/perl -w

$version = '$Id: bandlimit-failover.pl,v 1.9 2004/11/29 12:00:22 andrewm Exp $';
# (c) 2004 Leading Edge Business Solutions and Andrew McGill
# Distribution permitted under terms of GPL licence, as contained
# in the file COPYING
#
# Telkom bandlimit fail-over
#
# NOTE - DON'T RUN THIS AUTOMATICALLY EXCEPT FROM routeadjust.sh
# (you'll get strange behaviour)
#
# Telkom SA sells 3G of bandwidth for R250.00 (50USD).  After you
# pass this limit, you are bandlimited for "international"
# traffic.  This script checks whether you have passwd the limit
# and then brings up an additional interface on another ADSL
# account for international traffic only.
#
# Configuration files:
#
# /etc/ppp/pppoe.conf: main ppp0 account - never modified
# /etc/bandlimit-failover/account.conf: symbolic link to current account*.conf
# /etc/bandlimit-failover/account1.conf: first failover account for ppp1
# /etc/bandlimit-failover/account2.conf: second failover account for ppp1
# /etc/bandlimit-failover/account*.conf: additional failover accounts
#
# Usage:
# This script should be called in /etc/ppp/ip-up.d ... WITH A
# LOCK FILE ...  it is a real danger that the script will keep 
# bouncing links, especially if it runs multiple times.

no warnings qw{ uninitialized };

# We expect in our working directory a list of accounts
#
#	ppp0config	config for ppp0 - always in use
#	ppp1config	symbolic link to current ppp1 configuration - changed by this script
#	account1.conf	additional account details
#	account2.conf	additional account details
#	account3.conf	additional account details
#
# The account[123].conf files should share their PIDFILE setting
# to avoid running concurrently, and also so that account2 can
# shut down account1

$etc = "/etc/bandlimit-failover";
if ($ARGV[0] eq "--debug") {
	$DEBUG = "echo debug ...";
}
else {
	$DEBUG = '';
}

$ppp0config = "/etc/ppp/pppoe.conf";
$ppp1config = "$etc/active.conf";

# Set the file timestamp ...
sub touch($)
{
	utime undef,undef,(shift);
}

sub findnextaccount()
{
	@accountlist = sort glob("$etc/account*.conf");
	$activeaccount = readlink("$ppp1config");
	$nextaccount =  '';
	$usenext = 0;
	foreach $accountname (@accountlist) {
		if ($usenext) {
			$nextaccount = $accountname;
			$usenext = 0;
			last;
		}
		if ($activeaccount eq $accountname) {
			$usenext = 1;
		}
	}
	if ($nextaccount eq '' || $usenext) {
		# Rotate back to the first account
		$nextaccount = $accountlist[0];
	}
	system "ln -sf $nextaccount $ppp1config";
	($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
		$atime,$mtime,$ctime,$blksize,$blocks) = stat($nextaccount);
	$age = time() - $mtime;
	if ($age >= 0 && $age < 3600) {
		print "Skipping $ppp1config => $nextaccount (age $age seconds)\n";
		return 0;
	}
	else {
		print "Using config $ppp1config =>  $nextaccount\n";
		touch($nextaccount);
		return 1;
	}
}


# Get our IP
sub getpppipaddr($)
{
	my $interface = shift;
	my $look;
	@ifconfig = @ifconfig_orig;
	while ($_ = shift @ifconfig) {
		if ( m/^$interface\s/ ) { $look=1; }
		if ($look && m/inet addr:([\d.]+)/) {
			return $1;
		}
	}
	return '';
}


sub isbandlimited
{
	
	my @ppp= @_;
	# Telkom has 165.165.64.0/17 as their degraded network
	# Well - they did ...
	return (
		$ppp[0] == 165 && $ppp[1] == 165 && $ppp[2] <= 128
	);
}

sub checkbandlimited($)
{
	my $ppp = shift;
	my $pingpeer = "acknow.ledge.co.za";
	my $loss = 0;
	my @ping = qx{
		ping -q -c 5 -I $ppp $pingpeer 2>&1
	};
	# PING acknow.ledge.co.za (80.68.91.48) from 10.0.0.12 eth0: 56(84) bytes of data.
	#
	# --- acknow.ledge.co.za ping statistics ---
	# 5 packets transmitted, 5 received, 0% packet loss, time 4011ms
	# rtt min/avg/max/mdev = 208.179/216.199/239.561/11.736 ms

	while ( $_ = shift @ping ) {
		if ( m/(\d+)% packet loss\s/ ) { $loss=$1; }
		if ($look && m/inet addr:([\d.]+)/) {
			return $1;
		}
	}
	if ($loss > 40 && $loss < 100) {
		return 1;
	}
	else {
		return 0;
	}
}

# See what's up, what's down, and what's limited 
sub getpppstatus()
{
	@ifconfig_orig = qx{ ifconfig };
	$ppp0 = getpppipaddr("ppp0");
	@ppp0 = split /\./,$ppp0;
	$bandlimited0 = isbandlimited( @ppp0 ) || checkbandlimited("ppp0");
	$ppp0status = $ppp0[0] ? ( $bandlimited0 ? "bandlimited" : "normal" ) : "down";

	$ppp1 = getpppipaddr("ppp1");
	@ppp1 = split /\./,$ppp1;
	$bandlimited1 = isbandlimited ($ppp1) || checkbandlimited("ppp1");
	$ppp1status = $ppp1[0] ? ( $bandlimited1 ? "bandlimited" : "normal" ) : "down";

	# Check if there is a ppp2 up - there shouldn't be, but
	# it is possible
	#	$ppp2 = getpppipaddr("ppp2") || checkbandlimited("ppp2");
	#	@ppp2 = split /\./,$ppp2;
	#	$bandlimited2 = isbandlimited($ppp2);
	#	$ppp2status = $ppp2[0] ? ( $bandlimited2 ? "bandlimited" : "normal" ) : "down";
}


# These are the states that are catered for 
#
# 0: no connection
# 1: single connection (ppp0, not bandlimited)
# 2: degraded connection (ppp0 which is bandlimited)
# 3: dual connection (ppp0 bandlimited, ppp1 no limit)
# 4: degraded dual connection (ppp0 andlimited, ppp1 bandlimited)
#
# We rely on the internals of the rp-pppoe adsl-start script to
# ensure that we don't run more than one interface simultaneously
# - specifically that it names the lock file after the symbolic
# link, rather than what that link points to.

getpppstatus();

$version =~ s/^.Id: //;
$version =~ s/,v//;
$version =~ s/ \d+:\d\d:\d\d.*//;
print "$version\n";
print "Before: ppp0 $ppp0 $ppp0status, ppp1 $ppp1 $ppp1status\n";

if ($ppp0 eq '') {
	if ($ppp1 ne '') {
		print "Strange, ppp1 active without ppp0 ... killing it all\n";
		print "ACTION: taking down ".readlink($ppp1config)." and $ppp0config\n";
		system "$DEBUG adsl-stop $ppp1config";
		system "$DEBUG adsl-stop $ppp0config";
	}
	# no connection
	print "State: No ppp connection\n";
	print "ACTION: bring ppp0 up\n";
	system "$DEBUG adsl-start $ppp0config";
}
elsif ($ppp1 eq '') {
	# Single connection
	if ($bandlimited0) {
		# Degraded connecton
		if (findnextaccount()) {
			print "ACTION: starting ppp1\n";
			system "$DEBUG adsl-start $ppp1config";  # fixme ... this needs to refer to a config file ...
		}
		else {
			print "ACTION: Nothing! No untried accounts left\n";
		}
	}
	else {
		print "$DEBUG Single, ppp0 $ppp0, not bandlimited (no action)\n";
		exit;
	}
}
else { # Both ppp0 and ppp1 are up
	# Dual connection
	if (! $bandlimited0) {
		print "ACTION: taking unnecessary ppp1 down\n";
		system "$DEBUG adsl-stop $ppp1config"; 
	}
	elsif ($bandlimited1) {
		# ppp1 has run out of juice - rotate to next link
		print "ACTION: rotating ".readlink($ppp1config)."\n";
		system "$DEBUG adsl-stop $ppp1config";

		if (findnextaccount()) {
			sleep(5);
			print "ACTION: starting ppp1\n";
			system "$DEBUG adsl-start $ppp1config";
			sleep(10); # don't be in a hurry ...
		}
		else {
			print "ACTION: Nothing! No accounts available!\n";
		}
	}
	else {
		print "Action: No change\n";
		exit; # Don't print status 
	}
}

getpppstatus();
print "After: ppp0 $ppp0 $ppp0status, ppp1 $ppp1 $ppp1status\n";
